package com.example.samsungpay;

import android.content.Context;
import android.text.TextUtils;
import android.util.Log;

import com.samsung.android.sdk.samsungpay.v2.payment.sheet.AmountBoxControl;
import com.samsung.android.sdk.samsungpay.v2.payment.sheet.AmountConstants;
import com.samsung.android.sdk.samsungpay.v2.payment.sheet.CustomSheet;

import java.util.Locale;

public class AmountDetailControls {

    private static final String TAG = "AmountDetailControls";
    private Context mContext;

    private static final String AMOUNT_CONTROL_ID = "amountControlId";
    private static final String PRODUCT_ITEM_ID = "productItemId";
    private static final String PRODUCT_TAX_ID = "productTaxId";
    private static final String PRODUCT_SHIPPING_ID = "productShippingId";
    private static final String PRODUCT_FUEL_ID = "productFuelId";

    private String mProductPrice;
    private String mProductPriceDec;
    private String mTaxPrice;
    private String mTaxPriceDec;
    private String mShippingPrice;
    private String mShippingPriceDec;

    private String mCurrency;

    private double mDiscountedProductAmount;
    private double mTaxAmount ;
    private double mShippingAmount;
    private double mAddedShippingAmount;
    private double mAddedBillingAmount;

    AmountDetailControls(Context context,double mDiscountedProductAmount,
                         double mTaxAmount,
                         double mShippingAmount,
                         double mAddedShippingAmount,
                         double mAddedBillingAmount,
                         String currency) {
        mContext = context;
        mCurrency=currency;
        init( mDiscountedProductAmount,
         mTaxAmount,
         mShippingAmount,
         mAddedShippingAmount,
         mAddedBillingAmount);
    }

    private void init(double mDiscountedProductAmount,
                      double mTaxAmount,
                      double mShippingAmount,
                      double mAddedShippingAmount,
                      double mAddedBillingAmount) {

        this.mDiscountedProductAmount = mDiscountedProductAmount;
        this.mTaxAmount = mTaxAmount;
        this.mShippingAmount = mShippingAmount;
        this.mAddedShippingAmount = mAddedShippingAmount;
        this.mAddedBillingAmount = mAddedBillingAmount;

        mProductPrice=String.valueOf((int)mDiscountedProductAmount);
        mProductPriceDec="00";
        mTaxPrice=String.valueOf((int)mTaxAmount);
        mTaxPriceDec="00";
        mShippingPrice=String.valueOf((int)mShippingAmount);
        mShippingPriceDec="00";


    }

    private double getTotalAmount() {
        double mProductAmount = Double.parseDouble(getPriceValue(mProductPrice, mProductPriceDec));
        mTaxAmount = Double.parseDouble(getPriceValue(mTaxPrice, mTaxPriceDec));
        mShippingAmount = Double.parseDouble(getPriceValue(mShippingPrice, mShippingPriceDec));

        return mProductAmount + mTaxAmount + mAddedBillingAmount + mShippingAmount + mAddedShippingAmount;
    }

    private String getPriceValue(String price, String priceDec) {
        String left = "0";
        if (price != null) {
            String tmp = price;
            if (!TextUtils.isEmpty(tmp)) {
                left = tmp;
            }
        }

        String right = "00";
        if (priceDec != null) {
            String tmp = priceDec;
            if (!TextUtils.isEmpty(tmp)) {
                right = tmp;
            }
        }
        return left + "." + right;
    }


    private String getAmountFormat() {

        return AmountConstants.FORMAT_TOTAL_PRICE_ONLY;
    }
//
    AmountBoxControl makeAmountControl() {
        AmountBoxControl amountBoxControl = new AmountBoxControl(AMOUNT_CONTROL_ID, mCurrency);

        amountBoxControl.addItem(PRODUCT_ITEM_ID, mContext.getString(R.string.amount_control_name_item), mDiscountedProductAmount, "");
        amountBoxControl.addItem(PRODUCT_TAX_ID, mContext.getString(R.string.amount_control_name_tax), mTaxAmount + mAddedBillingAmount, "");
        amountBoxControl.addItem(PRODUCT_SHIPPING_ID, mContext.getString(R.string.amount_control_name_shipping), mShippingAmount + mAddedShippingAmount, "");
        amountBoxControl.setAmountTotal(getTotalAmount(), getAmountFormat());

        return amountBoxControl;
    }



    CustomSheet updateAmountControl(CustomSheet sheet) {
        AmountBoxControl amountBoxControl = (AmountBoxControl) sheet.getSheetControl(AMOUNT_CONTROL_ID);
        if (amountBoxControl == null) {
            Log.e(TAG, "updateAmountControl amountBoxControl : null");
            return sheet;
        }
        amountBoxControl.updateValue(PRODUCT_ITEM_ID, mDiscountedProductAmount);
        amountBoxControl.updateValue(PRODUCT_TAX_ID, mTaxAmount + mAddedBillingAmount);
        amountBoxControl.updateValue(PRODUCT_SHIPPING_ID, mShippingAmount + mAddedShippingAmount);
        if (!amountBoxControl.existItem(PRODUCT_FUEL_ID)) {
            amountBoxControl.addItem(3, PRODUCT_FUEL_ID, mContext.getString(R.string.amount_control_name_fuel), 0,
                    mContext.getString(R.string.amount_control_pending));
        } else {
            amountBoxControl.updateValue(PRODUCT_FUEL_ID, 0, mContext.getString(R.string.amount_control_pending));
        }

        amountBoxControl.setAmountTotal(getTotalAmount(), getAmountFormat());
        sheet.updateControl(amountBoxControl);

        return sheet;
    }
}
